import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_webrtc/flutter_webrtc.dart';
import 'package:videosdk/participant.dart';
import 'package:videosdk/src/features/me/bloc/me_bloc.dart';
import 'package:videosdk/src/features/peers/enitity/peer.dart';
import 'package:videosdk/src/features/room/bloc/room_bloc.dart';
import 'package:videosdk/src/features/signaling/room_client_repository.dart';
import 'package:events2/events2.dart';

class Meeting {
  final BuildContext context;
  late final EventEmitter _eventEmitter;

  Meeting(this.context) {
    this.localParticipant = new Participant(
      context,
      context.read<MeBloc>().state.id,
      context.read<MeBloc>().state.displayName,
      true,
    );

    this._eventEmitter = context.read<RoomClientRepository>().eventEmitter;

    this.participants = new Map();

    context.read<RoomClientRepository>().onMeetingJoined =
        (roomClientRepository) {
      // on meeting joined
    };

    _eventEmitter.on("peers-bloc-participant-joined", (Peer newPeer) {
      final newParticipant = new Participant(
        context,
        newPeer.id,
        newPeer.displayName,
        false,
      );
      participants[newPeer.id] = newParticipant;
      _eventEmitter.emit('participant-joined', newParticipant);
    });

    _eventEmitter.on("peers-bloc-participant-left", (peerId) {
      participants.remove(peerId);
      _eventEmitter.emit('participant-left', peerId);
    });

    _eventEmitter.on("peers-bloc-presenter-changed", (peerId) {
      context
          .read<RoomBloc>()
          .add(RoomSetActivePresenterId(presenterId: peerId));
    });
  }

  late Participant localParticipant;

  late Map<String, Participant> participants;

  String? get activeSpeakerId {
    return context.read<RoomBloc>().state.activeSpeakerId;
  }

  String? get activePresenterId {
    return context.read<RoomBloc>().state.activePresenterId;
  }

  void disableWebcam() {
    context.read<RoomClientRepository>().disableWebcam();
  }

  void enableWebcam() {
    context.read<RoomClientRepository>().enableWebcam();
  }

  void muteMic() {
    context.read<RoomClientRepository>().disableMic();
  }

  void unmuteMic() {
    context.read<RoomClientRepository>().enableMic();
  }

  void join() {
    context.read<RoomClientRepository>().join();
  }

  void leave() {
    context.read<RoomClientRepository>().close();
  }

  void startRecording(String webhookUrl) {
    context.read<RoomClientRepository>().startRecording(webhookUrl);
  }

  void stopRecording() {
    context.read<RoomClientRepository>().stopRecording();
  }

  void startLivestream(outputs) {
    context.read<RoomClientRepository>().startLivestream(outputs);
  }

  void stopLivestream() {
    context.read<RoomClientRepository>().stopLivestream();
  }

  void changeWebcam(String deviceId) {
    context.read<RoomClientRepository>().changeWebcam(deviceId);
  }

  List<MediaDeviceInfo> getWebcams() {
    return context.read<RoomClientRepository>().getWebcams();
  }

  String? get selectedWebcamId {
    return context.read<RoomClientRepository>().selectedWebcamId;
  }

  List<MediaDeviceInfo> getMics() {
    return context.read<RoomClientRepository>().getMics();
  }

  String? get selectedMicId {
    return context.read<RoomClientRepository>().selectedMicId;
  }

  on(String event, Function handler) {
    this._eventEmitter.on(event, handler);
  }
}
